@@ -2,16 +2,16 @@
  * TransFig: Facility for Translating Fig code
  * Copyright (c) 1991 by Micah Beck
  * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-1999 by Brian V. Smith
+ * Parts Copyright (c) 1989-2002 by Brian V. Smith
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.
+ * rights to use, copy, modify, merge, publish and/or distribute copies of
+ * the Software, and to permit persons who receive copies from any such 
+ * party to do so, with the only requirement being that this copyright 
+ * notice remain intact.
  *
  */
 
@@ -29,22 +29,14 @@
  *
  * 	Author Micah Beck, Cornell University, 4/88
  *	Color, rotated text and ISO-chars added by Herbert Bauer 11/91
+ *	PCL job control option added Brian V. Smith 1/2001
 */
 
-static set_style();
-
-#if defined(hpux) || defined(SYSV) || defined(SVR4)
-#include <sys/types.h>
-#endif
-#include <sys/file.h>
-#if defined(SYSV) || defined(SVR4)
-#include <string.h>
-#else
-#include <strings.h>
-#endif
 #include "fig2dev.h"
 #include "object.h"
 
+static set_style();
+
 #define		FONTS 			35
 #define		COLORS 			8
 #define		PATTERNS 		21
@@ -63,7 +55,9 @@ static	int	ibmgec		 = True;
 #else
 static	int	ibmgec		 = False;
 #endif
-static	int	reflected	 = False;
+
+static	Boolean	pcljcl		 = False;  /* flag to precede IBMGL (HP/GL) output with PCL job control */
+static	Boolean	reflected	 = False;
 static	int	fonts		 = FONTS;
 static	int	colors		 = COLORS;
 static	int	patterns	 = PATTERNS;
@@ -113,6 +107,10 @@ static	double	high[]		 = {.8,.8,.8,.8,.8,.8,.8,.8,.8,.8,.8,.8,.8,.8,
 static void genibmgl_option(opt, optarg)
 char opt, *optarg;
 {
+	FILE	*ffp;
+	int	 font;
+	int	 pattern;
+
 	switch (opt) {
 
 	    case 'a':				/* paper size		*/
@@ -133,39 +131,35 @@ char opt, *optarg;
 		break;
 
 	    case 'f':				/* user's characters	*/
-		{
-		    FILE	*ffp;
-		    int		font;
-		    if ((ffp = fopen(optarg, "r")) == NULL)
-			fprintf(stderr, "Couldn't open %s\n", optarg);
-		    else
-			for (font = 0; font <= fonts; font++)
-			    fscanf(ffp, "%d%d%lf%lf%lf",
+		if ((ffp = fopen(optarg, "r")) == NULL)
+		    fprintf(stderr, "Couldn't open %s\n", optarg);
+		else
+		    for (font = 0; font <= fonts; font++)
+			fscanf(ffp, "%d%d%lf%lf%lf",
 				&standard[font],	/* 0-4 6-9 30-39*/
 				&alternate[font],	/* 0-4 6-9 30-39*/
 				&slant[font],		/*   degrees	*/
 				&wide[font],		/*	~1.0	*/
 				&high[font]);		/*	~1.0	*/
-		    fclose(ffp);
-		}
+		fclose(ffp);
+		break;
+
+	    case 'k':				/* precede output with PCL job control */
+		pcljcl = True;
 		break;
 
 	    case 'l':				/* user's fill patterns	*/
-		{
-		    FILE	*ffp;
-		    int		pattern;
-		    if ((ffp = fopen(optarg, "r")) == NULL)
-			fprintf(stderr, "Couldn't open %s\n", optarg);
-		    else
-			for (pattern = 0; pattern < patterns; pattern++)
-			    fscanf(ffp, "%d%lf%d%lf%lf",
+		if ((ffp = fopen(optarg, "r")) == NULL)
+		    fprintf(stderr, "Couldn't open %s\n", optarg);
+		else
+		    for (pattern = 0; pattern < patterns; pattern++)
+			fscanf(ffp, "%d%lf%d%lf%lf",
 				&line_type[pattern],	/*    -1-6	*/
 				&line_space[pattern],	/*   inches	*/
 				&fill_type[pattern],	/*     1-5	*/
 				&fill_space[pattern],	/*   inches	*/
 				&fill_angle[pattern]);	/*   degrees	*/
-		    fclose(ffp);
-		}
+		fclose(ffp);
 		break;
 
 	    case 's':
@@ -267,11 +261,12 @@ F_compound	*objects;
 	    exit(1);
 	    }
 
-	ppi	 = objects->nwcorner.x;
 	cpi	 = mag*100.0/sqrt((xu-xl)*(xu-xl) + (yu-yl)*(yu-yl));
 	cpp	 = cpi/ppi;
 
 	/* IBMGL start */
+	if (pcljcl)
+	    fprintf(tfp,"\033%%0B\033%%E\n");	/* set to HP/GL mode and reset printer */
 	fprintf(tfp, "IN;\n");			/* initialize plotter	*/
 
 	if (!landscape) {			/* portrait mode	*/
@@ -430,27 +425,25 @@ double	length;
 		    break;
 		}
 	    }
-    }
+}
 
 /* 
  * set_width - issue line width commands as appropriate
  *		NOTE: for HP plotters we can't do anything
  */
 static set_width(w)
-int	w;
+    int	w;
 {
-static	int	line_width	 = DEFAULT;	/* in pixels		 */
-	line_width  = w;
-	}
+}
 
 /* 
  * set_color - issue line color commands as appropriate
  */
 static set_color(color)
-int	color;
+    int	color;
 {
-static	int	number		 = 0;	/* 1 <= number <= 8		*/
-static	double	thickness	 = 0.3;	/* pen thickness in millimeters	*/
+    static	int	number		 = 0;	/* 1 <= number <= 8		*/
+    static	double	thickness	 = 0.3;	/* pen thickness in millimeters	*/
 	if (line_color != color) {
 	    line_color  = color;
 	    color	= (colors + color)%colors;
@@ -463,11 +456,11 @@ static	double	thickness	 = 0.3;	/* pen thickness in millimeters	*/
 		fprintf(tfp, "PT%.4f;\n", pen_thickness[color]);
 		}
 	    }
-	}
+}
 
 static fill_polygon(pattern, color)
-int	pattern;
-int	color;
+    int	pattern;
+    int	color;
 {
 	if (0 < pattern && pattern < patterns) {
 	    int		style;
@@ -490,10 +483,10 @@ int	color;
 	    dash_length	 = DEFAULT;
 	    set_style(style, length);
 	    }
-	}
+}
 	
 void arc(sx, sy, cx, cy, theta, delta)
-double	sx, sy, cx, cy, theta, delta;
+    double	sx, sy, cx, cy, theta, delta;
 {
 	if (ibmgec)
 	    if (delta == M_PI/36.0)		/* 5 degrees		*/
@@ -517,10 +510,10 @@ double	sx, sy, cx, cy, theta, delta;
 	    	    cx + (sx - cx)*cos(theta) - (sy - cy)*sin(theta),
 	    	    cy + (sy - cy)*cos(theta) + (sx - cx)*sin(theta));
 	    }
-	}
+}
 
 void genibmgl_arc(a)
-F_arc	*a;
+    F_arc	*a;
 {
 	if (a->thickness != 0 ||
 		ibmgec && 0 <= a->fill_style && a->fill_style < patterns) {
@@ -575,10 +568,10 @@ F_arc	*a;
 	    if (0 < a->fill_style && a->fill_style < patterns)
 		fill_polygon(a->fill_style, a->fill_color);
 	    }
-	}
+}
 
 void genibmgl_ellipse(e)
-F_ellipse	*e;
+    F_ellipse	*e;
 {
 	if (e->thickness != 0 ||
 		ibmgec && 0 <= e->fill_style && e->fill_style < patterns) {
@@ -620,14 +613,19 @@ F_ellipse	*e;
 	    if (0 < e->fill_style && e->fill_style < patterns)
 		fill_polygon((int)e->fill_style, e->fill_color);
 	    }
-	}
+}
 
 void swap(i, j)
-int	*i, *j;
-{	int	t; t = *i; *i = *j; *j = t; }
+    int	*i, *j;
+{
+	int	t;
+	t = *i;
+	*i = *j;
+	*j = t;
+}
 
 void genibmgl_line(l)
-F_line	*l;
+    F_line	*l;
 {
 	if (l->thickness != 0 ||
 		ibmgec && 0 <= l->fill_style && l->fill_style < patterns) {
@@ -728,12 +726,12 @@ F_line	*l;
 		    break;
 		}
 	    }
-	}
+}
 
 #define		THRESHOLD	.05	/* inch */
 
 static bezier_spline(a0, b0, a1, b1, a2, b2, a3, b3)
-double	a0, b0, a1, b1, a2, b2, a3, b3;
+    double	a0, b0, a1, b1, a2, b2, a3, b3;
 {
 	double	x0, y0, x3, y3;
 	double	sx1, sy1, sx2, sy2, tx, ty, tx1, ty1, tx2, ty2, xmid, ymid;
@@ -754,7 +752,7 @@ double	a0, b0, a1, b1, a2, b2, a3, b3;
 	    bezier_spline(x0, y0, sx1, sy1, sx2, sy2, xmid, ymid);
 	    bezier_spline(xmid, ymid, tx1, ty1, tx2, ty2, x3, y3);
 	    }
-	}
+}
 
 static void genibmgl_itp_spline(s)
 F_spline	*s;
@@ -970,6 +968,7 @@ genibmgl_end()
 struct driver dev_ibmgl = {
      	genibmgl_option,
 	genibmgl_start,
+	gendev_null,
 	genibmgl_arc,
 	genibmgl_ellipse,
 	genibmgl_line,
