@@ -1,35 +1,38 @@
 /*
  * TransFig: Facility for Translating Fig code
- * Copyright (c) 1985 Supoj Sutantavibul
- * Copyright (c) 1991 Micah Beck
- * Parts Copyright (c) 1994 Brian V. Smith
+ * Copyright (c) 1991 by Micah Beck
+ * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-2002 by Brian V. Smith
  *
- * THE AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
- * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
- * EVENT SHALL THE AUTHORS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
- * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
- * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
- *
- * The X Consortium, and any party obtaining a copy of these files from
- * the X Consortium, directly or indirectly, is granted, free of charge, a
+ * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * rights to use, copy, modify, merge, publish and/or distribute copies of
+ * the Software, and to permit persons who receive copies from any such 
+ * party to do so, with the only requirement being that this copyright 
+ * notice remain intact.
+ *
  */
 
+
 /*
  *	genps.h: PostScript headers/patterns etc.
  *
  */
 
+extern Boolean	epsflag;	/* to distinguish PS and EPS */
+extern Boolean	pdfflag;	/* to distinguish PDF and PS/EPS */
+extern Boolean	tiffpreview;	/* add a TIFF preview? */
+
+extern void	genps_grid();
+extern void	genps_arc();
+extern void	genps_ellipse();
+extern void	genps_line();
+extern void	genps_spline();
+extern void	genps_text();
+
+
 #define		BEGIN_PROLOG1	"\
 /$F2psDict 200 dict def\n\
 $F2psDict begin\n\
@@ -148,7 +151,7 @@ $F2psDict /mtrx matrix put\n\
 	3 index 		% nw nh px py str nh\n\
 	{			% nw nh px py str\n\
 	  currentpoint		% nw nh px py str cx cy\n\
-	  2 index show		% nw nh px py str cx cy\n\
+	  2 index oldshow	% nw nh px py str cx cy\n\
 	  YStep add moveto	% nw nh px py str\n\
 	} repeat		% nw nh px py str\n\
     } for\n\
@@ -165,7 +168,7 @@ $F2psDict /mtrx matrix put\n\
   0 1 4 index length 2 sub {\n\
 				% string proc char idx\n\
     % Find the n+1th character in the string\n\
-    3 index exch 1 add get	% string proe char char+1\n\
+    3 index exch 1 add get	% string proc char char+1\n\
     exch 2 copy			% strinq proc char+1 char char+1 char\n\
     % Now show the nth character\n\
     PATsstr dup 0 4 -1 roll put	% string proc chr+1 chr chr+1 (chr)\n\
@@ -500,13 +503,10 @@ end\n\
 /Multi 2 def\n\
 /PaintData [\n\
   { clippath } bind\n\
-  { 32 32 true [ 32 0 0 -32 0 32 ]\n\
-	{<808080804040404020202020101010100808080804040404\n\
-	020202020101010180808080404040402020202010101010\n\
-	080808080404040402020202010101018080808040404040\n\
-	202020201010101008080808040404040202020201010101\n\
-	808080804040404020202020101010100808080804040404\n\
-	0202020201010101>}\n\
+  { 20 20 true [ 20 0 0 -20 0 20 ]\n\
+	{<8020004010002008001004000802000401000200\n\
+	8001004000802000401080200040100020080010\n\
+	0400080200040100020080010040008020004010>}\n\
      imagemask } bind\n\
 ] def\n\
 /PaintProc {\n\
@@ -531,13 +531,10 @@ end\n\
 /Multi 2 def\n\
 /PaintData [\n\
   { clippath } bind\n\
-  { 32 32 true [ 32 0 0 -32 0 32 ]\n\
-	{<010101010202020204040404080808081010101020202020\n\
-	404040408080808001010101020202020404040408080808\n\
-	101010102020202040404040808080800101010102020202\n\
-	040404040808080810101010202020204040404080808080\n\
-	010101010202020204040404080808081010101020202020\n\
-	4040404080808080>}\n\
+  { 20 20 true [ 20 0 0 -20 0 20 ]\n\
+	{<0040100080200100400200800401000802001004\n\
+	0020080040100080200000401000802001004002\n\
+	0080040100080200100400200800401000802000>}\n\
      imagemask } bind\n\
 ] def\n\
 /PaintProc {\n\
@@ -562,13 +559,10 @@ end\n\
 /Multi 2 def\n\
 /PaintData [\n\
   { clippath } bind\n\
-  { 32 32 true [ 32 0 0 -32 0 32 ]\n\
-	{<828282824444444428282828101010102828282844444444\n\
-	828282820101010182828282444444442828282810101010\n\
-	282828284444444482828282010101018282828244444444\n\
-	282828281010101028282828444444448282828201010101\n\
-	828282824444444428282828101010102828282844444444\n\
-	8282828201010101>}\n\
+  { 20 20 true [ 20 0 0 -20 0 20 ]\n\
+	{<8020004050102088201104400a02800401000a02\n\
+	8011044020882040501080200040501020882011\n\
+	04400a02800401000a0280110440208820405010>}\n\
      imagemask } bind\n\
 ] def\n\
 /PaintProc {\n\
@@ -1094,14 +1088,14 @@ newfontname newfont definefont pop end } def\n\
 /isovec [\n\
 "
 #define		SPECIAL_CHAR_2	"\
-8#200 /grave 8#201 /acute 8#202 /circumflex 8#203 /tilde\n\
+8#055 /minus 8#200 /grave 8#201 /acute 8#202 /circumflex 8#203 /tilde\n\
 8#204 /macron 8#205 /breve 8#206 /dotaccent 8#207 /dieresis\n\
 8#210 /ring 8#211 /cedilla 8#212 /hungarumlaut 8#213 /ogonek 8#214 /caron\n\
 8#220 /dotlessi 8#230 /oe 8#231 /OE\n\
 8#240 /space 8#241 /exclamdown 8#242 /cent 8#243 /sterling\n\
 8#244 /currency 8#245 /yen 8#246 /brokenbar 8#247 /section 8#250 /dieresis\n\
 8#251 /copyright 8#252 /ordfeminine 8#253 /guillemotleft 8#254 /logicalnot\n\
-8#255 /endash 8#256 /registered 8#257 /macron 8#260 /degree 8#261 /plusminus\n\
+8#255 /hyphen 8#256 /registered 8#257 /macron 8#260 /degree 8#261 /plusminus\n\
 8#262 /twosuperior 8#263 /threesuperior 8#264 /acute 8#265 /mu 8#266 /paragraph\n\
 8#267 /periodcentered 8#270 /cedilla 8#271 /onesuperior 8#272 /ordmasculine\n\
 8#273 /guillemotright 8#274 /onequarter 8#275 /onehalf\n\
@@ -1166,5 +1160,4 @@ newfontname newfont definefont pop end } def\n\
 #define		END_PROLOG	"\
 /$F2psBegin {$F2psDict begin /$F2psEnteredState save def} def\n\
 /$F2psEnd {$F2psEnteredState restore end} def\n\
-%%EndProlog\n\
 "
