@@ -1,15 +1,15 @@
 /*
  * TransFig: Facility for Translating Fig code
- * Copyright (c) 1989-1999 by Brian V. Smith
+ * Parts Copyright (c) 1989-2002 by Brian V. Smith
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.
+ * rights to use, copy, modify, merge, publish and/or distribute copies of
+ * the Software, and to permit persons who receive copies from any such 
+ * party to do so, with the only requirement being that this copyright 
+ * notice remain intact.
  *
  */
 
@@ -28,7 +28,8 @@
 #include "fig2dev.h"
 #include "object.h"
 
-extern	int	_read_pcx();
+extern	FILE	*open_picfile();
+extern	int	 _read_pcx();
 
 #define BUFLEN 1024
 
@@ -49,8 +50,6 @@ static Boolean	 ReadColorMap();
 static Boolean	 DoGIFextension();
 static int	 GetDataBlock();
 static int	 GetCode();
-extern FILE	*open_picfile();
-extern void	 close_picfile();
 
 #define LOCALCOLORMAP		0x80
 #define	ReadOK(file,buffer,len)	(fread(buffer, len, 1, file) != 0)
@@ -80,20 +79,27 @@ struct {
 */
 
 int
-read_gif(file,filetype,pic,llx,lly)
-    FILE	   *file;
+read_gif(filename,filetype,pic,llx,lly)
+    char	   *filename;
     int		    filetype;
     F_pic	   *pic;
     int		   *llx, *lly;
 {
-	char		buf[BUFLEN],pcxname[PATH_MAX];
-	FILE		*giftopcx;
-	int		i, stat, size;
-	int		useGlobalColormap;
-	unsigned char	localColorMap[3][MAXCOLORMAPSIZE];
-	unsigned int	bitPixel, red, green, blue;
-	unsigned char	c;
-	char		version[4];
+	char		 buf[BUFLEN],pcxname[PATH_MAX];
+	char		 realname[PATH_MAX];
+	FILE		*file, *giftopcx;
+	int		 i, stat, size;
+	int		 useGlobalColormap;
+	unsigned int	 bitPixel;
+	unsigned char	 c;
+	char		 version[4];
+	unsigned char    transp[3]; /* RGB of transparent color (if any) */
+
+	/* open the file */
+	if ((file=open_picfile(filename, &filetype, False, realname)) == NULL) {
+		fprintf(stderr,"No such GIF file: %s\n",realname);
+		return;
+	}
 
 	/* first read header to look for any transparent color extension */
 
@@ -111,7 +117,7 @@ read_gif(file,filetype,pic,llx,lly)
 	version[3] = '\0';
 
 	if ((strcmp(version, "87a") != 0) && (strcmp(version, "89a") != 0)) {
-		fprintf(stderr,"Unknown GIF version %s",version);
+		fprintf(stderr,"Unknown GIF version %s\n",version);
 		return 0;
 	}
 
@@ -126,7 +132,6 @@ read_gif(file,filetype,pic,llx,lly)
 	GifScreen.Background      = (unsigned int) buf[5];
 	GifScreen.AspectRatio     = (unsigned int) buf[6];
 
-	/* put in the width/height now in case there is some other failure later */
 	if (BitSet(buf[4], LOCALCOLORMAP)) {	/* Global Colormap */
 		if (!ReadColorMap(file,GifScreen.BitPixel,pic->cmap)) {
 			return 0;	/* error reading global colormap */
@@ -147,7 +152,7 @@ read_gif(file,filetype,pic,llx,lly)
 
 		if (c == '!') { 		/* Extension */
 		    if (! ReadOK(file,&c,1))
-			fprintf(stderr,"GIF read error on extention function code");
+			fprintf(stderr,"GIF read error on extention function code\n");
 		    (void) DoGIFextension(file, c);
 		    continue;
 		}
@@ -156,13 +161,17 @@ read_gif(file,filetype,pic,llx,lly)
 			continue;
 		}
 
+		if (! ReadOK(file,buf,9)) {
+			return 1;	/* couldn't read left/top/width/height */
+		}
+
 		useGlobalColormap = ! BitSet(buf[8], LOCALCOLORMAP);
 
 		bitPixel = 1<<((buf[8]&0x07)+1);
 
 		if (! useGlobalColormap) {
 		    if (!ReadColorMap(file, bitPixel, pic->cmap)) {
-			fprintf(stderr,"error reading local GIF colormap" );
+			fprintf(stderr,"error reading local GIF colormap\n" );
 			return 1;
 		    }
 		}
@@ -174,10 +183,15 @@ read_gif(file,filetype,pic,llx,lly)
 
 	/* save transparent indicator */
 	pic->transp = Gif89.transparent;
+	/* and RGB values */
+	if (pic->transp != -1) {
+	    transp[RED]   = pic->cmap[RED][pic->transp];
+	    transp[GREEN] = pic->cmap[GREEN][pic->transp];
+	    transp[BLUE]  = pic->cmap[BLUE][pic->transp];
+	}
 
 	/* reposition the file at the beginning */
-	close_picfile(file,filetype);
-	file = open_picfile(pic->file, &filetype, True, pcxname);
+	fseek(file, 0, SEEK_SET);
 	
 	/* now call giftopnm and ppmtopcx */
 
@@ -194,12 +208,14 @@ read_gif(file,filetype,pic,llx,lly)
 	}
 	/* close pipe */
 	pclose(giftopcx);
-	if ((giftopcx = fopen(pcxname, "r")) == NULL) {
+	if ((giftopcx = fopen(pcxname, "rb")) == NULL) {
 	    fprintf(stderr,"Can't open temp output file\n");
 	    return 0;
 	}
 	/* now call _read_pcx to read the pcx file */
 	stat = _read_pcx(giftopcx, pic);
+	/* close file */
+	fclose(giftopcx);
 
 	/* remove temp file */
 	unlink(pcxname);
@@ -207,19 +223,10 @@ read_gif(file,filetype,pic,llx,lly)
 	/* now match original transparent colortable index with possibly new 
 	   colortable from ppmtopcx */
 	if (pic->transp != -1) {
-	    if (useGlobalColormap) {
-		red = GifScreen.ColorMap[RED][pic->transp];
-		green = GifScreen.ColorMap[GREEN][pic->transp];
-		blue = GifScreen.ColorMap[BLUE][pic->transp];
-	    } else {
-		red = localColorMap[RED][pic->transp];
-		green = localColorMap[GREEN][pic->transp];
-		blue = localColorMap[BLUE][pic->transp];
-	    }
 	    for (i=0; i<pic->numcols; i++) {
-		if (pic->cmap[RED][i] == red &&
-		    pic->cmap[GREEN][i] == green &&
-		    pic->cmap[BLUE][i] == blue)
+		if (pic->cmap[RED][i]   == transp[RED] &&
+		    pic->cmap[GREEN][i] == transp[GREEN] &&
+		    pic->cmap[BLUE][i]  == transp[BLUE])
 			break;
 	    }
 	    if (i < pic->numcols)
@@ -240,7 +247,7 @@ unsigned char cmap[3][MAXCOLORMAPSIZE];
 
 	for (i = 0; i < number; ++i) {
 	    if (! ReadOK(fd, rgb, sizeof(rgb))) {
-		fprintf(stderr,"bad GIF colormap" );
+		fprintf(stderr,"bad GIF colormap\n" );
 		return False;
 	    }
 	    cmap[RED][i]   = rgb[RED];
@@ -259,19 +266,19 @@ int	label;
 	char	    *str;
 
 	switch (label) {
-	case 0x01:		/* Plain Text Extension */
+	    case 0x01:		/* Plain Text Extension */
 		str = "Plain Text Extension";
 		break;
-	case 0xff:		/* Application Extension */
+	    case 0xff:		/* Application Extension */
 		str = "Application Extension";
 		break;
-	case 0xfe:		/* Comment Extension */
+	    case 0xfe:		/* Comment Extension */
 		str = "Comment Extension";
 		while (GetDataBlock(fd, buf) != 0) {
 			; /* GIF comment */
 		}
 		return False;
-	case 0xf9:		/* Graphic Control Extension */
+	    case 0xf9:		/* Graphic Control Extension */
 		str = "Graphic Control Extension";
 		(void) GetDataBlock(fd, (unsigned char*) buf);
 		Gif89.disposal    = (buf[0] >> 2) & 0x7;
@@ -283,7 +290,7 @@ int	label;
 		while (GetDataBlock(fd, buf) != 0)
 			;
 		return False;
-	default:
+	    default:
 		str = (char *) buf;
 		sprintf(str, "UNKNOWN (0x%02x)", label);
 		break;
