@@ -1,27 +1,18 @@
 /*
  * TransFig: Facility for Translating Fig code
- * Copyright (c) 1985 Supoj Sutantavibul
- * Copyright (c) 1991 Micah Beck
+ * Copyright (c) 1991 by Micah Beck
+ * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-2002 by Brian V. Smith
  *
- * THE AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
- * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
- * EVENT SHALL THE AUTHORS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
- * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
- * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
- *
- * The X Consortium, and any party obtaining a copy of these files from
- * the X Consortium, directly or indirectly, is granted, free of charge, a
+ * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * rights to use, copy, modify, merge, publish and/or distribute copies of
+ * the Software, and to permit persons who receive copies from any such 
+ * party to do so, with the only requirement being that this copyright 
+ * notice remain intact.
+ *
  */
 
 /*
@@ -45,17 +36,16 @@ static	char	sccsfid[] = "@(#) getopt.c 5.0 (UTZoo) 1985";
 #define	ENDARGS  "--"
 
 /* this is included because index is not on some UNIX systems */
-static
-char *
+static char *
 index (s, c)
-register	char	*s;
-register	int 	c;
-	{
+  register  char *s;
+  register  int   c;
+{
 	while (*s)
 		if (c == *s) return (s);
 		else s++;
 	return (NULL);
-	}
+}
 
 /*
  * get option letter from argument vector
@@ -69,8 +59,8 @@ char	*optarg;		/* argument associated with option */
 		fputc(optopt,stderr);fputc('\n',stderr);return(BADCH);
 
 fig_getopt(nargc,nargv,ostr)
-int	nargc;
-char	**nargv,
+  int	nargc;
+  char	**nargv,
 	*ostr;
 {
 	static char	*place = EMSG;	/* option letter processing */
