@@ -2,23 +2,22 @@
  * TransFig: Facility for Translating Fig code
  * Copyright (c) 1991 by Micah Beck
  * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1994-1999 by Brian V. Smith
+ * Parts Copyright (c) 1989-2002 by Brian V. Smith
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.
+ * rights to use, copy, modify, merge, publish and/or distribute copies of
+ * the Software, and to permit persons who receive copies from any such 
+ * party to do so, with the only requirement being that this copyright 
+ * notice remain intact.
  *
  */
 
 #include <ctype.h>
 #include <errno.h>
 #include <sys/param.h>
-#include <string.h>
 #include "alloc.h"
 #include "fig2dev.h"
 #include "object.h"
@@ -33,7 +32,7 @@ strerror(e)
 	int e;
 {
 	return sys_errlist[e];
-	}
+}
 #endif
 
 extern int            errno;
@@ -58,9 +57,7 @@ static void		 count_lines_correctly();
 #define		BUF_SIZE	1024
 
 /* max number of comments that can be stored with each object */
-#define MAXCOMMENTS	50
-/* max length of any comment */
-#define MAXCOMLEN	200
+#define MAXCOMMENTS	100
 
 User_color	 user_colors[MAX_USR_COLS];
 int		 user_col_indx[MAX_USR_COLS];
@@ -103,7 +100,7 @@ int	err;
 	    }
 	else
 	    put_msg("File \"%s\" is not accessable; %s", file, strerror(err));
-	}
+}
 
 /**********************************************************
 Read_fig returns :
@@ -113,8 +110,7 @@ Read_fig returns :
     -2 : File is empty
 err_no : if file can not be read for various reasons
 
-The resolution (ppi) and the cooridnate system (coord_sys) are
-stored in obj->nwcorner.x and obj->nwcorner.x respectively.
+The resolution (ppi) is stored in global "ppi"
 **********************************************************/
 
 read_fig(file_name, obj)
@@ -123,12 +119,14 @@ F_compound	*obj;
 {
 	FILE		*fp;
 
+	/* initialize pattern_used[] array */
 	init_pats_used();
+
 	if ((fp = fopen(file_name, "r")) == NULL)
 	    return errno;
 	else
 	    return readfp_fig(fp, obj);
-	}
+}
 
 readfp_fig(fp, obj)
 FILE	*fp;
@@ -162,7 +160,7 @@ F_compound	*obj;
 	    status = read_1_3_objects(fp, obj);
 	(void)fclose(fp);
 	return status;
-	}
+}
 	
 int
 read_objects(fp, obj)
@@ -175,7 +173,7 @@ F_compound	*obj;
 	F_spline	*s, *ls = NULL;
 	F_arc		*a, *la = NULL;
 	F_compound	*c, *lc = NULL;
-	int		object, ppi, coord_sys, len;
+	int		object, coord_sys, len;
 
 	bzero((char*)obj, COMOBJ_SIZE);
 	(void) fgets(buf, BUF_SIZE, fp);	/* get the version line */
@@ -246,8 +244,7 @@ F_compound	*obj;
 		    return -1;
 		}
 		if (!paperspec) {
-		    /* copy all except newline */
-		    strncpy(papersize,buf,strlen(buf)-1);
+		    strcpy(papersize,buf);
 		    /* and truncate at first blank, if any */
 		    if (p=strchr(papersize,' '))
 			*p = '\0';
@@ -278,12 +275,10 @@ F_compound	*obj;
 		}
 		if (!transspec) {
 		    gif_colnum = atof(buf);
-		    if (gif_colnum < NUM_STD_COLS) {
+		    /* if standard color, get the name from the array */
+		    /* for user colors, wait till we've read in the file to get the value */
+		    if (gif_colnum < NUM_STD_COLS && gif_colnum >= 0)
 			strcpy(gif_transparent,Fig_color_names[gif_colnum]);
-		    } else {
-			/* set flag to get user color value once they've been read from file */
-			gif_colnum = -gif_colnum;
-		    }
 		}
 	    } 
 	} else {
@@ -301,12 +296,12 @@ F_compound	*obj;
 	    }
 	}
 
-	/* now read for resolution and coord_sys */
+	/* now read for resolution and coord_sys (coord_sys is not used) */
 	if (get_line(fp) < 0) {
 	    put_msg("File is truncated at resolution specification.");
 	    return -1;
 	    }
-	if (sscanf(buf,"%d%d\n", &ppi, &coord_sys) != 2) {
+	if (sscanf(buf,"%lf%d\n", &ppi, &coord_sys) != 2) {
 	    put_msg("Incomplete resolution information at line %d", line_no);
 	    return -1;
 	    }
@@ -316,8 +311,6 @@ F_compound	*obj;
 	/* attach any comments found thus far to the whole figure */
 	obj->comments = attach_comments();
 
-	obj->nwcorner.x = ppi;
-	obj->nwcorner.y = coord_sys;
 	while (get_line(fp) > 0) {
 	    if (sscanf(buf, "%d", &object) != 1) {
 		put_msg("Incorrect format at line %d", line_no);
@@ -422,10 +415,10 @@ F_compound	*obj;
 
 	/* if user color was requested for GIF transparent color, get the
 	   rgb values from the user color array now that we've read them in */
-	if (gif_colnum < 0) {
+	if (gif_colnum >= NUM_STD_COLS) {
 	    int i;
 	    for (i=0; i<num_usr_cols; i++)
-		if (user_col_indx[i] == -gif_colnum)
+		if (user_col_indx[i] == gif_colnum)
 		    break;
 	    if (i < num_usr_cols)
 		sprintf(gif_transparent,"#%2x%2x%2x", 
@@ -436,7 +429,8 @@ F_compound	*obj;
 	    return 0;
 	else
 	    return errno;
-	} /*  read_objects */
+
+} /*  read_objects */
 
 static void
 read_colordef(fp)
@@ -727,6 +721,7 @@ FILE	*fp;
 	int	type, style, radius_flag;
 	double	thickness, wid, ht;
 	int	npts;
+	char	file[PATH_MAX], *c;
 
 	Line_malloc(l);
 	l->points = NULL;
@@ -801,40 +796,34 @@ FILE	*fp;
 	    l->back_arrow = make_arrow(type, style, thickness, wid, ht);
 	}
     	if (l->type == T_PIC_BOX) {
-	  Pic_malloc(l->pic);
-	  if (l->pic  == NULL) {
-	    free((char *) l);
-	    return (NULL);
-	  }
-	  if (get_line(fp) < 0 || 
-	      sscanf(buf, "%d %s", &l->pic->flipped, l->pic->file) != 2) {
+	    Pic_malloc(l->pic);
+	    l->pic->transp = -1;
+	    if (l->pic  == NULL) {
+		free((char *) l);
+		return (NULL);
+	    }
+	    if (get_line(fp) < 0 || 
+	      sscanf(buf, "%d %s", &l->pic->flipped, file) != 2) {
 	        put_msg(Err_incomp,
 		    "Picture object", line_no);
 	        put_msg(Err_incomp, "Picture object", line_no);
 	        return (NULL);
-	  }
-
-#ifdef V4_0
-	  l->pic->figure=NULL;
-	  Compound_malloc(l->pic->figure);
-	  if (l->pic->figure == NULL) {
-	    free((char*) l);
-	    put_msg("No memory.");
-	    return (NULL);
-	  }
-	  
-	  if (push() == -1) 
-	      return (NULL);
-	  suppress_error=1;
-	  if (read_fig(l->pic->file,l->pic->figure) != 0) {
-	    free ((char *) l->pic->figure);
-	    l->pic->figure=NULL;
-	  }
-	  suppress_error=0;
-	  pop();
-#endif /* V4_0 */
-
-
+	    }
+	    /* if there is a path in the .fig filename, and the path of the imported
+	       picture filename is NOT absolute, prepend the .fig file path to it */
+	    if (from && file[0] != '/') {
+		/* copy the .fig filename to pic filename */
+		strcpy(l->pic->file, from);
+		if (c = strrchr(l->pic->file,'/')) {
+		    /* there is a '/', copy the filename past the last one */
+		    strcpy(c+1, file);
+		} else {
+		    /* either absolute picture file path or no path in the .fig filename */
+		    strcpy(l->pic->file, file);
+		}
+	    } else {
+		strcpy(l->pic->file, file);
+	    }
     	} else
 	   l->pic = NULL;
 	
@@ -875,18 +864,6 @@ FILE	*fp;
 	  p = q;
 	}
 
-#ifdef V4_0
-	if ((l->pic!=NULL)&&(l->pic->figure!=NULL)) {
-	    compound_bound(l->pic->figure,
-			&l->pic->figure->nwcorner.x,
-			&l->pic->figure->nwcorner.y,
-			&l->pic->figure->secorner.x,
-			&l->pic->figure->secorner.y);
-	    convert_figure(l);
-	  
-	}
-#endif /* V4_0 */
-	
 	l->comments = attach_comments();	/* attach any comments */
 	/* skip to the end of the line */
 	skip_line(fp);
@@ -1020,13 +997,17 @@ FILE	*fp;
 
 	    l = create_line_with_spline(s);
 	    free_splinestorage(s);  
+	    /* skip to end of line */
+	    skip_line(fp);
 	    if (l == NULL)
 		return NULL;
 	    return (F_spline *)l;   /* return the new line */
 	  }
 
-	if (approx_spline(s)) 
+	if (approx_spline(s)) {
+	    skip_line(fp);
 	    return s;
+	}
 	/* Read controls from older versions */
 	/* keep track of newlines for line counter */
 	count_lines_correctly(fp);
@@ -1091,7 +1072,7 @@ FILE	*fp;
 		&t->type, &t->color, &t->depth, &t->pen,
 		&t->font, &t->size, &t->angle,
 		&t->flags, &t->height, &t->length,
-		&t->base_x, &t->base_y, s, junk);
+		&t->base_x, &t->base_y, s);
 	} else {
 	    /* The text object is terminated by a CONTROL-A, so we read
 		everything up to the CONTROL-A and then read that character.
@@ -1110,7 +1091,7 @@ FILE	*fp;
  	  return NULL;
 	}
 
-	if (font_size) {
+	if (font_size != 0.0) {
 	    /* scale length/height of text by ratio of requested font size to actual size */
 	    t->length = t->length * font_size/t->size;
 	    t->height = t->height * font_size/t->size;
@@ -1324,7 +1305,7 @@ get_line(fp)
 	    buf[len-1] = '\0';			/* strip trailing newline */
 	    if (buf[len-2] == '\r')
 		buf[len-2] = '\0';		/* strip any trailing CRs */
-	    return (1);
+	    return 1;
 	}
     }
 }
@@ -1336,11 +1317,12 @@ save_comment(fp)
 {
     int		    i;
 
-    /* see if we've allocated space for this comment */
-    if (comments[numcom] == 0)
-	if ((comments[numcom] = malloc(MAXCOMLEN)) == NULL)
-	    return -1;
     i=strlen(buf);
+    /* see if we've allocated space for this comment */
+    if (comments[numcom])
+	free(comments[numcom]);
+    if ((comments[numcom] = malloc(i+1)) == NULL)
+	    return -1;
     /* remove any newline */
     if (buf[i-1] == '\n')
 	buf[i-1] = '\0';
@@ -1373,6 +1355,8 @@ int	 fill_style;
 	}
 }
 	
+/* reset pattern_used[] array */
+
 init_pats_used()
 {
 	int i;
