@@ -1,33 +1,23 @@
 /*
  * TransFig: Facility for Translating Fig code
- * Copyright (c) 1985 Supoj Sutantavibul
- * Copyright (c) 1991 Micah Beck
+ * Copyright (c) 1991 by Micah Beck
+ * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-2002 by Brian V. Smith
  *
- * THE AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
- * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
- * EVENT SHALL THE AUTHORS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
- * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
- * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
- *
- * The X Consortium, and any party obtaining a copy of these files from
- * the X Consortium, directly or indirectly, is granted, free of charge, a
+ * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * rights to use, copy, modify, merge, publish and/or distribute copies of
+ * the Software, and to permit persons who receive copies from any such 
+ * party to do so, with the only requirement being that this copyright 
+ * notice remain intact.
+ *
  */
 
 /*******************************************************************/
 /***************       Read version 1.3 format       ***************/
 /*******************************************************************/
-#include <stdio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <errno.h>
@@ -51,10 +41,6 @@
 
 extern F_arrow		*forward_arrow(), *backward_arrow();
 extern int		figure_modified;
-#ifndef __NetBSD__
-extern char		*sys_errlist[];
-extern int		sys_nerr, errno;
-#endif
 
 static F_ellipse	*read_ellipseobject();
 static F_line		*read_lineobject();
@@ -66,6 +52,10 @@ static F_compound	*read_compoundobject();
 extern int		line_no;
 extern int		num_object;
 
+#ifdef V4_0
+extern int              suppress_error;/*ggstemme*/
+#endif /* V4_0 */
+
 int
 read_1_3_objects(fp, obj)
 FILE		*fp;
@@ -81,10 +71,13 @@ F_compound	*obj;
 	int		 object, pixperinch, canvaswid, canvasht, coord_sys;
 
 	n = fscanf(fp,"%d%d%d%d\n", &pixperinch, &coord_sys, &canvaswid, &canvasht);
-	if (n != 4) {
-	    put_msg("Incorrect format in the first line in input file");
-	    return(-1);
-	    }
+	if (n != 4){
+#ifdef V4_0
+	  if (!suppress_error)
+#endif /* V4_0 */
+	     put_msg("Incorrect format in the first line in input file");
+	  return(-1);
+	}
 	obj->nwcorner.x = pixperinch;
 	obj->nwcorner.y = coord_sys;
 	while (fscanf(fp, "%d", &object) == 1) {
@@ -283,7 +276,7 @@ FILE	*fp;
 	if (feof(fp))
 	    return(com);
 	else {
-	    put_msg("Format error: %s", sys_errlist[errno]);
+	    put_msg("Format error: %s", strerror(errno));
 	    return(NULL);
 	    }
 	}
